static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/gofprofile/src/GofProfile.cpp,v 1.2 2020-02-20 09:02:44 giulio Exp $";
//+=============================================================================
//
// file :         GofProfile.cpp
//
// description :  C++ source for the GofProfile and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                GofProfile are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: giulio $ Giulio Gaio
//
// $Revision: 1.2 $ 1.0
//
// $Log: GofProfile.cpp,v $
// Revision 1.2  2020-02-20 09:02:44  giulio
// Added MeanOrbitHor MeanOrbitVer
//
// Revision 1.1.1.1  2014-10-06 12:24:46  giulio
// First cvs release
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name   |  Method's name
//	----------------------------------------
//  State            |  dev_state()
//  Status           |  dev_status()
//  GetSrCorrNames   |  get_sr_corr_names()
//  GetSrBpmNames    |  get_sr_bpm_names()
//  GetBpmPosition   |  get_bpm_position()
//  GetGofBpmNames   |  get_gof_bpm_names()
//  GetGofCorrNames  |  get_gof_corr_names()
//
//===================================================================


#include <tango.h>
#include <GofProfile.h>
#include <GofProfileClass.h>

#include <iostream>
#include <list>     
#include <map>
#include <math.h>

#include <gof_struct.h>
#include <math_lib.h>

extern unsigned long shm_dev;
extern unsigned long mbf_shm_dev;

namespace GofProfile_ns
{

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::GofProfile(string &s)
// 
// description : 	constructor for simulated GofProfile
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
GofProfile::GofProfile(Tango::DeviceClass *cl,string &s)
:Tango::Device_4Impl(cl,s.c_str())
{
	init_device();
}

GofProfile::GofProfile(Tango::DeviceClass *cl,const char *s)
:Tango::Device_4Impl(cl,s)
{
	init_device();
}

GofProfile::GofProfile(Tango::DeviceClass *cl,const char *s,const char *d)
:Tango::Device_4Impl(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		GofProfile::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void GofProfile::delete_device()
{
	//	Delete device's allocated object

	delete attr_MeanOrbitHor_read;
	delete attr_MeanOrbitVer_read;
	delete attr_AbsRmsOrbitHor_read;
	delete attr_AbsRmsOrbitVer_read;

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void GofProfile::init_device()
{
	INFO_STREAM << "GofProfile::GofProfile() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	FILE* fd;
	char buff[256],buf1[56],buf2[200];
	char module_str[12] = "gofprofile";
  int found = 0;
	
	/* Check if gofprofile module is installed */
	if ((fd = fopen("/proc/modules", "r")) < 0) {
		cout << "Gofprofile::init_devices: error opening procfs" << endl;
		Tango::Except::throw_exception (
			(const char *)"Proc filesystem not found",
			(const char *)"Proc filesystem not supported",
			(const char *)"Gofprofile::init_device()");
  }
	while (!feof(fd)) {
 		if (!fgets(buff, 256, fd)) {
			break;
		}
    sscanf(buff,"%s %s",buf1,buf2);
		if (strncmp(buf1,module_str,10) == 0) {	
			fclose(fd);
			found = 1;	
			printf("Gofprofile::init_devices: found %s module\n",module_str);
			break;
		}
	}
	
	if (!found) {	
		printf("Gofprofile::init_devices: module %s not found\n",module_str);
		fclose(fd);
		Tango::Except::throw_exception (
			(const char *)"Device not found",
			(const char *)"Gofprofile module not installed",
			(const char *)"Gofprofile::init_device()");	
	}
	else {
		// Shared memory initialized in main, get here addresses 			
		gofprofilem = (gofprofile_shm_t *) shm_dev;
		mbfm = (mbf_shm_t *) mbf_shm_dev;		
		if(gofprofilem == 0) {
			Tango::Except::throw_exception (
				(const char *)"OUT OF MEMORY",
				(const char *)"out of memory error, cannot allocate DEV rtai shared memory",
				(const char *)"Gofprofile::init_device()");
		}		
		if(mbfm == 0) {
			Tango::Except::throw_exception (
				(const char *)"OUT OF MEMORY",
				(const char *)"out of memory error, cannot allocate DEV rtai shared memory",
				(const char *)"Gofprofile::init_device()");
		}
		
	}	
	
	attr_MeanOrbitHor_read = new Tango::DevFloat();
	attr_MeanOrbitVer_read = new Tango::DevFloat();

	attr_AbsRmsOrbitHor_read = new Tango::DevFloat();
	attr_AbsRmsOrbitVer_read = new Tango::DevFloat();

	/* set frequency acq */
	gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff = (int) (10000/(float)slowFreq);
	gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff = (int) (10000/(float)fastFreq);
		
	try {
		get_bpm_corr_names();
	}
		catch(Tango::DevFailed e) {
		cout << "Failed to get bpm and corrector names from database" << endl;	 			
	}
	
	
}


//+----------------------------------------------------------------------------
//
// method : 		GofProfile::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void GofProfile::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("BpmExcluded"));
	dev_prop.push_back(Tango::DbDatum("CorrExcluded"));
	dev_prop.push_back(Tango::DbDatum("HistoryLen"));
	dev_prop.push_back(Tango::DbDatum("FastHistoryLen"));
	dev_prop.push_back(Tango::DbDatum("SlowFreq"));
	dev_prop.push_back(Tango::DbDatum("FastFreq"));
	dev_prop.push_back(Tango::DbDatum("RmsLen"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	GofProfileClass	*ds_class =
		(static_cast<GofProfileClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize BpmExcluded from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  bpmExcluded;
	//	Try to initialize BpmExcluded from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  bpmExcluded;
	//	And try to extract BpmExcluded value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  bpmExcluded;

	//	Try to initialize CorrExcluded from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  corrExcluded;
	//	Try to initialize CorrExcluded from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  corrExcluded;
	//	And try to extract CorrExcluded value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  corrExcluded;

	//	Try to initialize HistoryLen from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  historyLen;
	//	Try to initialize HistoryLen from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  historyLen;
	//	And try to extract HistoryLen value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  historyLen;

	//	Try to initialize FastHistoryLen from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  fastHistoryLen;
	//	Try to initialize FastHistoryLen from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  fastHistoryLen;
	//	And try to extract FastHistoryLen value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  fastHistoryLen;

	//	Try to initialize SlowFreq from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  slowFreq;
	//	Try to initialize SlowFreq from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  slowFreq;
	//	And try to extract SlowFreq value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  slowFreq;

	//	Try to initialize FastFreq from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  fastFreq;
	//	Try to initialize FastFreq from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  fastFreq;
	//	And try to extract FastFreq value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  fastFreq;

	//	Try to initialize RmsLen from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rmsLen;
	//	Try to initialize RmsLen from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  rmsLen;
	//	And try to extract RmsLen value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rmsLen;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		GofProfile::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void GofProfile::always_executed_hook()
{
	set_DevStat(); 
}
//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void GofProfile::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "GofProfile::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}
//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrHorMaxPerc
// 
// description : 	Extract real attribute values for CorrHorMaxPerc acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrHorMaxPerc(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrHorMaxPerc(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	double max_corr = 0;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] = (short)((float)
			(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[idx][corr_section_idx[i]].hor_corr[corr_corr_idx[i]]) / avg);
		if (abs(corr_array[i]) > max_corr)
			max_corr = (double) abs(corr_array[i]);
	}			
	
	attr_f_value = max_corr/32768*100;
	attr.set_value(&attr_f_value);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrVerMaxPerc
// 
// description : 	Extract real attribute values for CorrVerMaxPerc acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrVerMaxPerc(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrVerMaxPerc(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	double max_corr = 0;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] = (short)((float)
			(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[idx][corr_section_idx[i]].ver_corr[corr_corr_idx[i]]) / avg);
		if (abs(corr_array[i]) > max_corr)
			max_corr = (double) abs(corr_array[i]);
	}			
	
	attr_f_value = max_corr/32768*100;
	attr.set_value(&attr_f_value);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_MeanRmsOrbitHor
// 
// description : 	Extract real attribute values for MeanRmsOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_MeanRmsOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_MeanRmsOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_bpm; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	avg = 0;
	for (int i = 0; i < num_bpm; i++)
		avg += bpm_array_small[i];
	attr_f_value = avg/(float)num_bpm/1000;	

	attr.set_value(&attr_f_value);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_MeanRmsOrbitVer
// 
// description : 	Extract real attribute values for MeanRmsOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_MeanRmsOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_MeanRmsOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_bpm; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] =
(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	avg = 0;
	for (int i = 0; i < num_bpm; i++)
		avg += bpm_array_small[i];
	attr_f_value = avg/(float)num_bpm/1000;	

	attr.set_value(&attr_f_value);


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_ErrRmsOrbitHor
// 
// description : 	Extract real attribute values for ErrRmsOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_ErrRmsOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_ErrRmsOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else {	
			bpm_array[i] =
(float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm / avg - 
			(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]) ;
		}
	}				
	
	attr_f_value = pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,bpmstorageringnames.size()), bpmstorageringnames.size()),0.5)/1000;
	
	attr.set_value(&attr_f_value);

}


//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_AbsRmsOrbitHor
// 
// description : 	Extract real attribute values for AbsRmsOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_AbsRmsOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_AbsRmsOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else {	
			bpm_array[i] =
(float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm / avg) ;
		}
	}				
	
	*attr_AbsRmsOrbitHor_read = pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,bpmstorageringnames.size()), bpmstorageringnames.size()),0.5)/1000000;

	attr.set_value(attr_AbsRmsOrbitHor_read);

}


//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_MeanOrbitHor
// 
// description : 	Extract real attribute values for MeanOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_MeanOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_MeanOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;
	
	*attr_MeanOrbitHor_read = 0;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				Tango::Except::throw_exception(
					(const char *) "Error acquiring bpm",
					(const char *) "Gigabit error",
					(const char *) "Gofprofile::read_MeanOrbitHor()",Tango::ERR);
		}
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE) {
				Tango::Except::throw_exception(
					(const char *) "Error acquiring bpm",
					(const char *) "Overrange error",
					(const char *) "Gofprofile::read_MeanOrbitHor()",Tango::ERR);
		}
		else {	
			*attr_MeanOrbitHor_read +=
(float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm / avg) ;
		}
	}				

	*attr_MeanOrbitHor_read /= 1000000;

	attr.set_value(attr_MeanOrbitHor_read);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_ErrRmsOrbitVer
// 
// description : 	Extract real attribute values for ErrRmsOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_ErrRmsOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_ErrRmsOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else {	
			bpm_array[i] =
(float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm / avg - 
			(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]) ;
		}
	}				
	
	attr_f_value = pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,bpmstorageringnames.size()), bpmstorageringnames.size()),0.5)/1000;
	
	attr.set_value(&attr_f_value);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_AbsRmsOrbitVer
// 
// description : 	Extract real attribute values for AbsRmsOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_AbsRmsOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_AbsRmsOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else {	
			bpm_array[i] =
(float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm / avg) ;
		}
	}				
	
	*attr_AbsRmsOrbitVer_read = pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,bpmstorageringnames.size()), bpmstorageringnames.size()),0.5)/1000000;

	attr.set_value(attr_AbsRmsOrbitVer_read);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_MeanOrbitVer
// 
// description : 	Extract real attribute values for MeanOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_MeanOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_MeanOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;
	
	*attr_MeanOrbitVer_read = 0;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				Tango::Except::throw_exception(
					(const char *) "Error acquiring bpm",
					(const char *) "Gigabit error",
					(const char *) "Gofprofile::read_MeanOrbitVer()",Tango::ERR);
		}
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE) {
				Tango::Except::throw_exception(
					(const char *) "Error acquiring bpm",
					(const char *) "Overrange error",
					(const char *) "Gofprofile::read_MeanOrbitVer()",Tango::ERR);
		}
		else {	
			*attr_MeanOrbitVer_read +=
(float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm / avg) ;
		}
	}				

	*attr_MeanOrbitVer_read /= 1000000;

	attr.set_value(attr_MeanOrbitVer_read);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_GofRelativeOrbitHor
// 
// description : 	Extract real attribute values for GofRelativeOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_GofRelativeOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_GofRelativeOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;
	
	for (int i = 0; i < GOF_NUM_SECTION; i++) {
		for (int j = 0; j < GOF_NUM_BPM_SECTION; j++) {		
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].giga_status & (1 << j))
				bpm_array[i*GOF_NUM_SECTION+j] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].hor_bpm ==
				GOF_BPM_OVERRANGE)
				bpm_array[i*GOF_NUM_SECTION+j] = NAN;						
			else {	
				bpm_array[i*GOF_NUM_SECTION+j] =
				(float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].hor_bpm / avg - 
				(float)mbfm->param.ref_orbit_hor.rfo[i*GOF_NUM_SECTION+j]) ;
			}
		}
	}			
	
	attr_GofRelativeOrbitHor_read = &bpm_array[0];

	attr.set_value(attr_GofRelativeOrbitHor_read,GOF_NUM_SECTION*GOF_NUM_BPM_SECTION,0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_GofRelativeOrbitVer
// 
// description : 	Extract real attribute values for GofRelativeOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_GofRelativeOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_GofRelativeOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;
	
	for (int i = 0; i < GOF_NUM_SECTION; i++) {
		for (int j = 0; j < GOF_NUM_BPM_SECTION; j++) {		
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].giga_status & (1 << j))
				bpm_array[i*GOF_NUM_SECTION+j] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].ver_bpm ==
				GOF_BPM_OVERRANGE)
				bpm_array[i*GOF_NUM_SECTION+j] = NAN;						
			else {	
				bpm_array[i*GOF_NUM_SECTION+j] =
				(float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].ver_bpm / avg - 
				(float)mbfm->param.ref_orbit_ver.rfo[i*GOF_NUM_SECTION+j]) ;
			}
		}
	}			
	
	attr_GofRelativeOrbitVer_read = &bpm_array[0];

	attr.set_value(attr_GofRelativeOrbitVer_read,GOF_NUM_SECTION*GOF_NUM_BPM_SECTION,0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitHorHistory
// 
// description : 	Extract real attribute values for OrbitHorHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitHorHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitHorHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	{
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
			}
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
		}			
		row++;
	}

	attr_OrbitHorHistory_read = &bpm_array[0];

	attr.set_value(attr_OrbitHorHistory_read,historyLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitVerHistory
// 
// description : 	Extract real attribute values for OrbitVerHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitVerHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitVerHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	{
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);
			}
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);
		}			
		row++;
	}

	attr_OrbitVerHistory_read = &bpm_array[0];

	attr.set_value(attr_OrbitVerHistory_read,historyLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitHorHistoryFast
// 
// description : 	Extract real attribute values for OrbitHorHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitHorHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitHorHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
		}			
		row++;
	}

	attr_OrbitHorHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_OrbitHorHistoryFast_read,fastHistoryLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitVerHistoryFast
// 
// description : 	Extract real attribute values for OrbitVerHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitVerHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitVerHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg );
		}			
		row++;
	}

	attr_OrbitVerHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_OrbitVerHistoryFast_read,fastHistoryLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RmsOrbitHor
// 
// description : 	Extract real attribute values for RmsOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RmsOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RmsOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_bpm; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	attr_RmsOrbitHor_read = &bpm_array_small[0];
	attr.set_value(attr_RmsOrbitHor_read,num_bpm);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RmsOrbitVer
// 
// description : 	Extract real attribute values for RmsOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RmsOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RmsOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_bpm; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
				bpm_array_small[i] = NAN;
				break;
			}	
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE) {
				bpm_array_small[i] = NAN;
				break;
			}						
			else	
				bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	attr_RmsOrbitVer_read = &bpm_array_small[0];
	attr.set_value(attr_RmsOrbitVer_read,num_bpm);


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RmsCorrHor
// 
// description : 	Extract real attribute values for RmsCorrHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RmsCorrHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RmsCorrHor(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_corr; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][bpm_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	attr_RmsCorrHor_read = &bpm_array_small[0];
	attr.set_value(attr_RmsCorrHor_read,num_corr);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RmsCorrVer
// 
// description : 	Extract real attribute values for RmsCorrVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RmsCorrVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RmsCorrVer(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg,mean;
	
	int data_len = rmsLen*fastFreq;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - data_len) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (data_len - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - data_len);
		end1 = idx;
	}
	
	for (int i = 0; i < num_corr; i++) {
		row=0;
		bpm_array_small[i] = 0;
		for (int j = start1; j < end1; j++)	{
			bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);
			row++;
		}			
		for (int j = start2; j < end2; j++)	{
			bpm_array[row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][bpm_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);			
			row++;
		}
		
		if (bpm_array_small[i] == 0)
			bpm_array_small[i]=pow((double)math_var_f(bpm_array, math_mean_f(bpm_array,row), row),0.5);
	
	}

	attr_RmsCorrVer_read = &bpm_array_small[0];
	attr.set_value(attr_RmsCorrVer_read,num_corr);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrHorHistory
// 
// description : 	Extract real attribute values for CorrHorHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrHorHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrHorHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_corr; i++) {
			bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[j][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_corr; i++) {
			bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[j][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	attr_CorrHorHistory_read = &bpm_array[0];

	attr.set_value(attr_CorrHorHistory_read,historyLen,num_corr); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrVerHistory
// 
// description : 	Extract real attribute values for CorrVerHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrVerHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrVerHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_corr; i++) {
			bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[j][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_corr; i++) {
			bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[j][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	attr_CorrVerHistory_read = &bpm_array[0];

	attr.set_value(attr_CorrVerHistory_read,historyLen,num_corr); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrHorHistoryFast
// 
// description : 	Extract real attribute values for CorrHorHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrHorHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrHorHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_corr; i++) {	
			bpm_array[i*fastHistoryLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_corr; i++) {	
			bpm_array[i*fastHistoryLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	attr_CorrHorHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_CorrHorHistoryFast_read,fastHistoryLen,num_corr); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrVerHistoryFast
// 
// description : 	Extract real attribute values for CorrVerHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrVerHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrVerHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,corr,cnt = 0, idx, num_corr;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_corr = corrstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}
	
	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_corr; i++) {				
			bpm_array[i*fastHistoryLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_corr; i++) {		
			bpm_array[i*fastHistoryLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[j][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/avg);
		}			
		row++;
	}

	attr_CorrVerHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_CorrVerHistoryFast_read,fastHistoryLen,num_corr); /* historyLen = row */

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitHorHistory
// 
// description : 	Extract real attribute values for RelativeOrbitHorHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitHorHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitHorHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	{
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg - 
				(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
			}
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/avg - 
				(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	attr_RelativeOrbitHorHistory_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitHorHistory_read,historyLen,num_bpm); /* historyLen = row */

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitVerHistory
// 
// description : 	Extract real attribute values for RelativeOrbitVerHistory acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitVerHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitVerHistory(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;	
	
	if ((idx - historyLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (historyLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - historyLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	{
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg - 
				(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
			}
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*historyLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*historyLen+row] = NAN;						
			else	
				bpm_array[i*historyLen+row] =
				(float)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/avg - 
				(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	attr_RelativeOrbitVerHistory_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitVerHistory_read,historyLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitHorHistoryFast
// 
// description : 	Extract real attribute values for RelativeOrbitHorHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitHorHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitHorHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)
				/ avg - 
				(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm) / avg - 
				(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	attr_RelativeOrbitHorHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitHorHistoryFast_read,fastHistoryLen,num_bpm); /* historyLen = row */

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitVerHistoryFast
// 
// description : 	Extract real attribute values for RelativeOrbitVerHistoryFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitVerHistoryFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitVerHistoryFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0, idx, num_bpm;
	int start1 = 0,
	    end1 = 0,
			start2 = 0,
			end2 = 0;
	int row = 0;
	float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	num_bpm = bpmstorageringnames.size();
	
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;	
	
	if ((idx - fastHistoryLen) < 0) {
	  start1 = GOFPROFILE_STORE_SIZE - (fastHistoryLen - idx);
		end1 = GOFPROFILE_STORE_SIZE;
		start2 = 0;
		end2 = idx;
	}	 	
	else {
		start1 = (idx - fastHistoryLen);
		end1 = idx;
	}

	for (int j = start1; j < end1; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)
				/ avg - 
				(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	for (int j = start2; j < end2; j++)	{
		for (int i = 0; i < num_bpm; i++) {
			if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
				bpm_array[i*fastHistoryLen+row] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
					GOF_BPM_OVERRANGE)
				bpm_array[i*fastHistoryLen+row] = NAN;						
			else	
				bpm_array[i*fastHistoryLen+row] = (float)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[j][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm) / avg - 
				(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
		}			
		row++;
	}

	attr_RelativeOrbitVerHistoryFast_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitVerHistoryFast_read,fastHistoryLen,num_bpm); /* historyLen = row */


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitHor
// 
// description : 	Extract real attribute values for RelativeOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else {	
			bpm_array[i] =
(float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm / avg - 
			(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]) ;
		}
	}			
	
	attr_RelativeOrbitHor_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitHor_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitVer
// 
// description : 	Extract real attribute values for RelativeOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else	
			bpm_array[i] = (float)((float)gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm  / avg- 
			(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
	}			
	
	attr_RelativeOrbitVer_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitVer_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitHorFast
// 
// description : 	Extract real attribute values for RelativeOrbitHorFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitHorFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitHorFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else	
			bpm_array[i] = (float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm/ avg - 
			(float)mbfm->param.ref_orbit_hor.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]) ;
	}			
	
	attr_RelativeOrbitHorFast_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitHorFast_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_RelativeOrbitVerFast
// 
// description : 	Extract real attribute values for RelativeOrbitVerFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_RelativeOrbitVerFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_RelativeOrbitVerFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else	
			bpm_array[i] = (float)((float)gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm / avg - 
			(float)mbfm->param.ref_orbit_ver.rfo[bpm_section_idx[i]*GOF_NUM_SECTION+bpm_bpm_idx[i]]);
	}			
	
	attr_RelativeOrbitVerFast_read = &bpm_array[0];

	attr.set_value(attr_RelativeOrbitVerFast_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_GofOrbitHor
// 
// description : 	Extract real attribute values for GofOrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_GofOrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_GofOrbitHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < GOF_NUM_SECTION; i++) {
		for (int j = 0; j < GOF_NUM_BPM_SECTION; j++) {	
			if (bpmgofnames[i*GOF_NUM_BPM_SECTION+j] == "")
				bpm_array[cnt] = NAN;	
			else if (bpm_excluded_flag[i*GOF_NUM_BPM_SECTION+j])
				bpm_array[cnt] = NAN;				
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].giga_status & (1 << j))
				bpm_array[cnt] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].hor_bpm == GOF_BPM_OVERRANGE)
				bpm_array[cnt] = NAN;						
			else	
				bpm_array[cnt] = (float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].hor_bpm) / avg;
			cnt++;
		}			
	}
	
	attr_GofOrbitHor_read = &bpm_array[0];

	attr.set_value(attr_GofOrbitHor_read,GOF_NUM_SECTION * GOF_NUM_BPM_SECTION,0,false);


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_GofOrbitVer
// 
// description : 	Extract real attribute values for GofOrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_GofOrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_GofOrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < GOF_NUM_SECTION; i++) {
		for (int j = 0; j < GOF_NUM_BPM_SECTION; j++) {	
			if (bpmgofnames[i*GOF_NUM_BPM_SECTION+j] == "")
				bpm_array[cnt] = NAN;					
			else if (bpm_excluded_flag[i*GOF_NUM_BPM_SECTION+j])
				bpm_array[cnt] = NAN;	
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].giga_status & (1 << j))
				bpm_array[cnt] = NAN;
			else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].ver_bpm == GOF_BPM_OVERRANGE)
				bpm_array[cnt] = NAN;						
			else	
				bpm_array[cnt] = (float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][i].libera[j].ver_bpm)/ avg;
			cnt++;
		}			
	}
	
	attr_GofOrbitVer_read = &bpm_array[0];

	attr.set_value(attr_GofOrbitVer_read,GOF_NUM_SECTION * GOF_NUM_BPM_SECTION,0,false);

}
//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitHor
// 
// description : 	Extract real attribute values for OrbitHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitHor(Tango::Attribute &attr) entering... "<< endl;
	
	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;


	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;						
		else	{
			bpm_array[i] = (float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm) / avg;
	
		}			
	}
	attr_OrbitHor_read = &bpm_array[0];

	attr.set_value(attr_OrbitHor_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitVer
// 
// description : 	Extract real attribute values for OrbitVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;		
		else	
			bpm_array[i] = (float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/ avg;
	}			
	
	attr_OrbitVer_read = &bpm_array[0];

	attr.set_value(attr_OrbitVer_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrHor
// 
// description : 	Extract real attribute values for CorrHor acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrHor(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrHor(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] = (short)((float)
	
(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[idx][corr_section_idx[i]].hor_corr[corr_corr_idx[i]]) / avg);
	}			
	
	attr_CorrHor_read = &corr_array[0];

	attr.set_value(attr_CorrHor_read,corrstorageringnames.size(),0,false);


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrVer
// 
// description : 	Extract real attribute values for CorrVer acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrVer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrVer(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] =
	
(short)((float)(gofprofilem->arr_profile[GOFPROFILE_SLOW_IDX].storage.section_corr[idx][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/
avg);	
	}			
	
	attr_CorrVer_read = &corr_array[0];

	attr.set_value(attr_CorrVer_read,corrstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitHorFast
// 
// description : 	Extract real attribute values for OrbitHorFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitHorFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitHorFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i])) {
			bpm_array[i] = NAN;
		}	
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;		
		else	
			bpm_array[i] = (float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].hor_bpm)/ avg;
	}			
	
	attr_OrbitHorFast_read = &bpm_array[0];

	attr.set_value(attr_OrbitHorFast_read,bpmstorageringnames.size(),0,false);


}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_OrbitVerFast
// 
// description : 	Extract real attribute values for OrbitVerFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_OrbitVerFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_OrbitVerFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].giga_status & (1 << bpm_bpm_idx[i]))
			bpm_array[i] = NAN;
		else if (gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm ==
		GOF_BPM_OVERRANGE)
			bpm_array[i] = NAN;		
		else	
			bpm_array[i] = (float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_bpm[idx][bpm_section_idx[i]].libera[bpm_bpm_idx[i]].ver_bpm)/ avg;
	}				
	
	attr_OrbitVerFast_read = &bpm_array[0];

	attr.set_value(attr_OrbitVerFast_read,bpmstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrHorFast
// 
// description : 	Extract real attribute values for CorrHorFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrHorFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrHorFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] =
	
(short)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[idx][corr_section_idx[i]].hor_corr[corr_corr_idx[i]])/
avg);	
	}			
	
	attr_CorrHorFast_read = &corr_array[0];

	attr.set_value(attr_CorrHorFast_read,corrstorageringnames.size(),0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		GofProfile::read_CorrVerFast
// 
// description : 	Extract real attribute values for CorrVerFast acquisition result.
//
//-----------------------------------------------------------------------------
void GofProfile::read_CorrVerFast(Tango::Attribute &attr)
{
	DEBUG_STREAM << "GofProfile::read_CorrVerFast(Tango::Attribute &attr) entering... "<< endl;

	int section,bpm,cnt = 0,idx;float avg;
	
	idx = gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].store_counter;
	avg = (float) gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].average_coeff;

	for (int i = 0; i < corrstorageringnames.size(); i++) {
		corr_array[i] =
	
(short)((float)(gofprofilem->arr_profile[GOFPROFILE_FAST_IDX].storage.section_corr[idx][corr_section_idx[i]].ver_corr[corr_corr_idx[i]])/
avg);	
	}			
	
	attr_CorrVerFast_read = &corr_array[0];

	attr.set_value(attr_CorrVerFast_read,corrstorageringnames.size(),0,false);

}

//+------------------------------------------------------------------
/**
 *	method:	GofProfile::get_sr_corr_names
 *
 *	description:	method to execute "GetSrCorrNames"
 *	Get corrector names ordered by position in the storage ring
 *
 * @return	array of corrector names
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *GofProfile::get_sr_corr_names()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();

	DEBUG_STREAM << "GofProfile::get_sr_corr_names(): entering... !" << endl;

	//	Add your own code to control device here
	argout->length((long)corrstorageringnames.size());
	
	for (int i = 0; i < argout->length(); i++)
		(*argout)[i] = CORBA::string_dup(corrstorageringnames[i].c_str());

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	GofProfile::get_sr_bpm_names
 *
 *	description:	method to execute "GetSrBpmNames"
 *	Get bpm names ordered by position in the storage ring
 *
 * @return	array of bpm names
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *GofProfile::get_sr_bpm_names()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();

	DEBUG_STREAM << "GofProfile::get_sr_bpm_names(): entering... !" << endl;

	//	Add your own code to control device here
	argout->length((long)bpmstorageringnames.size());
	
	for (int i = 0; i < argout->length(); i++)
		(*argout)[i] = CORBA::string_dup(bpmstorageringnames[i].c_str());

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	GofProfile::get_bpm_position
 *
 *	description:	method to execute "GetBpmPosition"
 *	Get longitudinal bpm position in mm starting from section 1.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevVarLongArray *GofProfile::get_bpm_position()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarLongArray	*argout  = new Tango::DevVarLongArray();

	DEBUG_STREAM << "GofProfile::get_bpm_position(): entering... !" << endl;

	//	Add your own code to control device here
	argout->length((long)bpmstorageringposition.size());

	for (int i = 0; i < argout->length(); i++)
		(*argout)[i] = bpmstorageringposition[i];	
		
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Gofprofile::get_bpm_corr_names()
 *
 *	Get storage and gof address of the bpms and correctors 
 *
 *
 */
//+------------------------------------------------------------------
void GofProfile::get_bpm_corr_names()
{
	DEBUG_STREAM << "Gofprofile::get_bpm_corr_names(): entering... !" << endl;

	//	Add your own code to control device here
	using namespace Tango; 
	char count;
	int gof_idx, storagering_idx;
	vector<string>::iterator nums_iter;
	vector<long>::iterator nums_iter_l;
	Database *db = new Database();	

	bpmgofnames.resize(0);
	bpmstorageringnames.resize(0);
	bpmstorageringposition.resize(0);			
	corrgofnames.resize(0);
	corrstorageringnames.resize(0);

	bpmgofnames.resize(GOF_NUM_BPM_SECTION*GOF_NUM_SECTION);
	bpmstorageringnames.resize(GOF_NUM_BPM_SECTION*GOF_NUM_SECTION);
	bpmstorageringposition.resize(GOF_NUM_BPM_SECTION*GOF_NUM_SECTION);			
	corrgofnames.resize(GOF_NUM_CORR_SECTION*GOF_NUM_SECTION);
	corrstorageringnames.resize(GOF_NUM_CORR_SECTION*GOF_NUM_SECTION);
		
	for (int i = 0; i < GOF_NUM_BPM_SECTION*GOF_NUM_SECTION; i++)
		bpmstorageringposition[i] = -1;
		
	DbData db_data_bpm;
	db_data_bpm.push_back(DbDatum("position"));
	db->get_property("BPM",db_data_bpm);

	if (db_data_bpm[0].is_empty()) {
		Tango::Except::throw_exception(
    (const char *) "Cant'find bpm device in database",
		(const char *) "No bpm devices found",
		(const char *) "Gofprofile::get_bpm_corr_names()",Tango::ERR);		
	}
	else {
		
		long gofaddress_bpm[2],storageringaddress_bpm[2];
		vector<string> device_names_bpm;
		string bpmname;
		long position;
		
		db_data_bpm[0] >> device_names_bpm;

		for (vector<string>::const_iterator it = device_names_bpm.begin(); it < device_names_bpm.end(); it++) {
			
			std::string token;
			std::istringstream iss(*it);
			count = 0;
			while ( getline(iss, token, ':') ) {
				switch (count) {
					case 0: bpmname = token; break;
					case 1: position = atol(token.c_str());break; /* position S (mm) of bpm in the storage ring */ 
					case 2: storageringaddress_bpm[0] = atol(token.c_str());break; /* storage ring section (0..11) */				
					case 3: storageringaddress_bpm[1] = atol(token.c_str());break; /* bpm index inside the sr section (0..11) */	
					case 4: gofaddress_bpm[0] = atol(token.c_str());break; /* global feedback section (0..11) */				
					case 5: gofaddress_bpm[1] = atol(token.c_str());break; /* bpm index inside the global feedback section (0..11) */
				}
				count++;
				if (count >= 6)
					break;										
   		}
		
			if (count < 6) {
				Tango::Except::throw_exception(
    		(const char *) "Corrupted data in database",
				(const char *) "Invalid BPM position format",
				(const char *) "Gofprofile::get_bpm_corr_names()",Tango::ERR);		
			}					
		
			char found_flag = 0;
			for (int i = 0; i < bpmExcluded.size(); i++) {
				if (bpmname == bpmExcluded[i]) {
					found_flag = 1;
					break;
				}	
			}
		

			/* rfm2g data reflects gof addressing */
			gof_idx = gofaddress_bpm[0]*GOF_NUM_BPM_SECTION+gofaddress_bpm[1];
			storagering_idx = storageringaddress_bpm[0]*GOF_NUM_BPM_SECTION+storageringaddress_bpm[1];	
	
			/* assign a hash key to bpm name */
			bpmname_key[bpmname.c_str()] = 	gofaddress_bpm[1] + (gofaddress_bpm[0] << 16) + 0x10000000; 
			/* 0x10000000 is an existing bit flag */		

			bpmgofnames[gof_idx] = bpmname; /* names ordered by position */

			if (!found_flag) {
				/* insert bpm name in  profile vectors */
				bpmstorageringposition[storagering_idx] = position;
				bpmstorageringnames[storagering_idx] = bpmname;	
				bpm_excluded_flag[gof_idx] = 0;
			}
			else
				bpm_excluded_flag[gof_idx] = 1;			

		}				
		
		do {
			nums_iter = find(bpmstorageringnames.begin(), bpmstorageringnames.end(), "");
			if (nums_iter != bpmstorageringnames.end())
				bpmstorageringnames.erase(nums_iter,nums_iter+1); 
		} while (nums_iter != bpmstorageringnames.end());
		
		do {
			nums_iter_l = find(bpmstorageringposition.begin(), bpmstorageringposition.end(), -1);
			if (nums_iter_l != bpmstorageringposition.end())
				bpmstorageringposition.erase(nums_iter_l,nums_iter_l+1); 
		} while (nums_iter_l != bpmstorageringposition.end());
	
	}	

	DbData db_data_corr;
	db_data_corr.push_back(DbDatum("position"));
	db->get_property("Correctors",db_data_corr);

	if (db_data_corr[0].is_empty()) {
		Tango::Except::throw_exception(
    (const char *) "Cant'find corr device in database",
		(const char *) "No corr devices found",
		(const char *) "Gofprofile::get_bpm_corr_names()",Tango::ERR);		
	}
	else {
		vector<string> device_names_corr;
		long gofaddress_corr[2],storageringaddress_corr[2];
		string corrname;
		
		db_data_corr[0] >> device_names_corr;
	
		for (vector<string>::const_iterator it = device_names_corr.begin(); it < device_names_corr.end(); it++) {
			std::string token;
			std::istringstream iss(*it);
			count = 0;
			
			while ( getline(iss, token, ':') ) {
				switch (count) {
					case 0: corrname = token; break;
					case 1: storageringaddress_corr[0] = atol(token.c_str());break; /* storage ring section (0..11) */				
					case 2: storageringaddress_corr[1] = atol(token.c_str());break; /* corr index inside the sr section (0..11) */	
					case 3: gofaddress_corr[0] = atol(token.c_str());break; /* global feedback section (0..11) */				
					case 4: gofaddress_corr[1] = atol(token.c_str());break; /* corr index inside the global feedback section (0..11) */
				}
				count++;
				if (count >= 5)
					break;										
   		}
		
			if (count < 5) {
				Tango::Except::throw_exception(			
    		(const char *) "Corrupted data in database",
				(const char *) "Invalid corrector format",
				(const char *) "Gofprofile::get_bpm_corr_names()",Tango::ERR);		
			}		

			char found_flag = 0;
			for (int i = 0; i < corrExcluded.size(); i++) {
				if (corrname == corrExcluded[i]) {
					found_flag = 1;
					break;
				}	
			}
		

			/* rfm2g data reflects gof addressing */
			gof_idx = gofaddress_corr[0]*GOF_NUM_CORR_SECTION+gofaddress_corr[1];
			storagering_idx = storageringaddress_corr[0]*GOF_NUM_CORR_SECTION+storageringaddress_corr[1];
			/* assign a hash key to corr name */
			corrname_key[corrname.c_str()] = 	gofaddress_corr[1] + (gofaddress_corr[0] << 16) + 0x10000000; 
			/* 0x10000000 is an existing bit flag */		

			/* insert corr name in  profile vectors */
			corrgofnames[gof_idx] = corrname;			

			if (!found_flag) {		
				corrstorageringnames[storagering_idx] = corrname;
			}		

		}			

		do {
			nums_iter = find(corrstorageringnames.begin(), corrstorageringnames.end(), "");
			if (nums_iter != corrstorageringnames.end())
				corrstorageringnames.erase(nums_iter,nums_iter+1); 
		} while (nums_iter != corrstorageringnames.end());		
	
	
	}

	/* now create permanent profile indexes for fast data retreiving */
	int bpm_cnt = 0,
	    corr_cnt = 0;
	long tmp;
	
	for (int i = 0; i < bpmstorageringnames.size(); i++) {
		tmp = bpmname_key[bpmstorageringnames[i].c_str()] & 0xfffffff;
		bpm_section_idx[bpm_cnt] = tmp >> 16;
		bpm_bpm_idx[bpm_cnt] = tmp & 0xffff;		
		bpm_cnt++;
	}	 
	
	for (int i = 0; i < corrstorageringnames.size(); i++) {
		tmp = corrname_key[corrstorageringnames[i].c_str()] & 0xfffffff;
		corr_section_idx[corr_cnt] = tmp >> 16;
		corr_corr_idx[corr_cnt] = tmp & 0xffff;		
		corr_cnt++;	
	}	 
	
	delete db;		
	
}

//+------------------------------------------------------------------
/**
 *	method:	GofProfile::get_gof_bpm_names
 *
 *	description:	method to execute "GetGofBpmNames"
 *	Get bpm names ordered by global feedback logic
 *
 * @return	array of bpm names
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *GofProfile::get_gof_bpm_names()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();

	DEBUG_STREAM << "GofProfile::get_gof_bpm_names(): entering... !" << endl;

	//	Add your own code to control device here
	argout->length((long)bpmgofnames.size());
	
	for (int i = 0; i < argout->length(); i++)
		(*argout)[i] = CORBA::string_dup(bpmgofnames[i].c_str());
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	GofProfile::get_gof_corr_names
 *
 *	description:	method to execute "GetGofCorrNames"
 *	Get corrector names ordered by global feedback logic
 *
 * @return	array of correctors
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *GofProfile::get_gof_corr_names()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();

	DEBUG_STREAM << "GofProfile::get_gof_corr_names(): entering... !" << endl;

	//	Add your own code to control device here

	argout->length((long)corrgofnames.size());
	
	for (int i = 0; i < argout->length(); i++)
		(*argout)[i] = CORBA::string_dup(corrgofnames[i].c_str());
	return argout;

}



//+------------------------------------------------------------------
/**
 *	method:	GofProfile::set_DevStat
 *
 *	Convert Tango DevState DevStatus according to status masks 
 */
//+------------------------------------------------------------------
long GofProfile::set_DevStat(void) 
{
	long tmp = 0;
	
	std::stringstream s;

	tmp = mbfm->status;
	
	/* bit 9 is loop ON,OFF status */	
	if (mbfm->gof_status & 0x1)
	 	tmp |= MBF_GOFRUN;
	
	/*
	bit 0: 1=set acquisition ON; 0=set acquisition OFF
	bit 1: 1=status acquisition ON; 0=status acquisition OFF
	bit 2: 1=clock not found
	bit 3: 1=searching clock
	bit 4: 1=set save into buffer ON; 0=set save into buffer OFF
	bit 5: 1=status save into buffer ON; 0=status save into buffer OFF
	bit 6: 1=stop if error detected, 0=don't stop if error detected 
	bit 7: 1=automatic restart, 0=manual restart
	bit 8: 1=gof error, 0=ok
	bit 9: 1=gof running,  0=gof off 
	*/
	
	/*
	if (tmp & MBF_GOFRUN) {
		set_state(Tango::ON);
		s << "Global feedback ON" << endl;
	}
	else {
		set_state(Tango::ON);
		s << "Global feedback OFF" << endl;	
	}
	*/

	if ((tmp & MBF_CURBUFACQ) == 0) {
		s << "Stopped data acquisition on circular buffers" << endl;
		set_state(Tango::OFF);
	}
	else {
		s << "Acquisition running" << endl;
		set_state(Tango::ON);
	}

	if (mbfm->param.loop.freeze_flag & GOF_FREEZE_DAC) {
		tmp |= MBF_FREEZE;
		if (tmp & MBF_GOFRUN)
			set_state(Tango::FAULT);
		s << "Global feedback is FREEZED" << endl;		
	}
	else
		tmp &= ~MBF_FREEZE;

	if ((tmp & MBF_CURACQ) == 0) {
		set_state(Tango::FAULT);
		s << "Stopped acquisition from GOF slave stations" << endl;
	}	

	if ((tmp & MBF_CLKERR)) {
		set_state(Tango::FAULT);
		s << "Can't find 10kHz clock source" << endl;
	}	
	
	if ((tmp & MBF_GOFERR)) {
		set_state(Tango::FAULT);
		s << "Global feedback error" << endl;
	}

	/* if there is no beam and there are no hardware errors, put gof in disable status */
	if ((tmp & MBF_SUMERR) && (!(tmp & (MBF_CLKERR | MBF_RFM2GERR | MBF_BPMERR | MBF_QERR | MBF_CORRERR)))) {
		set_state(Tango::FAULT);
		s << "No BEAM" << endl;	
	}

	set_status(s.str());
	return tmp;

}


}	//	namespace
